@@ -560,7 +560,120 @@ def acker(A,B,poles):
     K = K[-1][:]                # Extract the last row            # Extract the last row
     
     return K    
+
+def right2df(system):
     
+    eig,V = np.linalg.eig(system.A)
+    W = np.linalg.inv(V)
+    N_x = W.shape[0]
+
+    modules = np.abs(V)
+    degs = np.rad2deg(np.angle(V))
+    W_row = []
+    W_list =[]
+    N_x = W.shape[0]
+    for irow in range(N_x):
+        W_row = []
+        for icol in range(N_x):
+            W_row += [f'{modules[icol,irow]:0.2f}∠{degs[icol,irow]:5.1f}']
+        W_list += [W_row]
+
+    modes = [f'Mode {it+1}' for it in range(N_x)]
+
+    df_report = pd.DataFrame(data=W_list,index=modes, columns=system.x_list)
+    df = pd.DataFrame(data=modules*np.exp(1j*np.angle(W)),index=modes, columns=system.x_list)
+    
+    system.shape_modules = modules
+    system.shape_degs = degs
+    system.modes_id = modes
+    system.df = df
+    
+    return df_report
+
+def left2df(system):
+    
+    eig,V = np.linalg.eig(system.A)
+    W = np.linalg.inv(V)
+    N_x = W.shape[0]
+
+    modules = np.abs(W)
+    degs = np.rad2deg(np.angle(W))
+    W_row = []
+    W_list =[]
+    N_x = W.shape[0]
+    for irow in range(N_x):
+        W_row = []
+        for icol in range(N_x):
+            W_row += [f'{modules[irow,icol]:0.2f}∠{degs[irow,icol]:5.1f}']
+        W_list += [W_row]
+
+    modes = [f'Mode {it+1}' for it in range(N_x)]
+
+    df_report = pd.DataFrame(data=W_list,index=modes, columns=system.x_list)
+    df = pd.DataFrame(data=modules*np.exp(1j*np.angle(W)),index=modes, columns=system.x_list)
+    
+    system.shape_modules = modules
+    system.shape_degs = degs
+    system.modes_id = modes
+    system.df = df
+    
+    return df_report
+
+
+def pi_design(a,b,zeta,omega):
+    '''
+    G = 1/(a*s + b)    
+    PI = (K_p + K_i/s) = (K_p*s + K_i)/s -> tf([K_p,K_i],[1,0])
+    K_i = K_p/T_p
+    T_p = K_p/K_i
+    
+    Example:
+    
+    L = 4e-3
+    R = 0.1
+
+    K_p,K_i = pi_design(L,R,1.0,2*np.pi*10)
+
+    G_planta = ctrl.tf([1], [L,R])
+    G_pi = ctrl.tf([K_p,K_i],[1,0])
+
+    H = G_planta*G_pi/(1 + G_planta*G_pi)
+    ctrl.bode_plot(H);
+    
+    '''
+    
+    K_p = 2*zeta*omega*a-b
+    K_i = a*omega**2
+    
+    return K_p,K_i
+
+def lead_design(angle,omega, verbose=False):
+    '''
+    Desing lead lag compensator: G = (T_1*s + 1)/(T_2*s + 1)
+    
+    alpha = (1+np.sin(angle))/(1-np.sin(angle))
+    
+    T_2 = 1/(omega*np.sqrt(alpha))
+    T_1 = alpha*T_2
+    
+
+    G_ll = ctrl.tf([T_1,1],[T_2,1])
+
+    ctrl.bode_plot(G_ll);
+    
+    '''
+    
+    alpha = (1+np.sin(angle))/(1-np.sin(angle))
+    T_2 = 1/(omega*np.sqrt(alpha))
+    T_1 = alpha*T_2
+    
+    if verbose:
+        G_omega = (T_1*1j*omega+1)/(T_2*1j*omega+1)
+        print(f'Gain at omega: {np.abs(G_omega)}')
+    
+    return T_1,T_2
+    
+     
 if __name__ == "__main__":
     
     class sys_class():
