@@ -77,14 +77,30 @@ class svg():
                     #    rect.attrib['style'].update(new_style)
                     #else:
                     rect.attrib['style'] = new_style
-                
+
+    def set_path_style(self,object_id,new_style):
+        #style="fill:#337ab7"
+        for path in self.root.findall('.//{http://www.w3.org/2000/svg}path'):
+            if 'id' in path.attrib:
+                if path.attrib['id'] == object_id: 
+                    #if 'style' in rect.attrib:
+                    #    rect.attrib['style'].update(new_style)
+                    #else:
+                    path.attrib['style'] = new_style
+                    print(path.attrib['style'])
+
+
     def set_color(self,type_,id_,rgb):
         if type_ == 'rect':
             self.set_rect_style(id_,f"fill:#{rgb[0]:02x}{rgb[1]:02x}{rgb[2]:02x}")
             
         if type_ == 'line':
             self.set_line_style(id_,f"stroke:#{rgb[0]:02x}{rgb[1]:02x}{rgb[2]:02x}")
-            
+
+        if type_ == 'path':
+            self.set_path_style(id_,f"stroke:#{rgb[0]:02x}{rgb[1]:02x}{rgb[2]:02x}")
+
+
     def tostring(self):
         self.svg_str = ET.tostring(self.root).decode()
         return self.svg_str
@@ -294,6 +310,9 @@ class svg():
             vscs_list = []
             
         for vsc in vscs_list:
+
+            if not 'bus_ac' in vsc: continue
+
             bus_ac = vsc['bus_ac']
             bus_dc = vsc['bus_dc']
             vsc_id = f'vsc_{bus_ac}_{bus_dc}'
